package jp.veritrans.mpos.sample.integration.ui;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CompoundButton;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.ComponentActivity;

import jp.veritrans.mpos.sample.integration.R;
import jp.veritrans.mpos.sample.integration.ui.support.IntegrationMessageContext;
import jp.veritrans.mpos.sample.integration.util.IntentUtil;
import jp.veritrans.mpos.v2.integration.MposIntegrationAction;
import jp.veritrans.mpos.v2.integration.MposIntegrationKey;
import jp.veritrans.mpos.v2.integration.MposIntegrationService;

/**
 * mPOSアプリの取消処理と連携するフラグメント。
 */
public class CancelFragment extends IntegrationBaseFragment
        implements View.OnClickListener, CompoundButton.OnCheckedChangeListener {

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        super.onCreateView(inflater, container, savedInstanceState);

        return inflater.inflate(R.layout.fragment_cancel, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        Button btn = view.findViewById(R.id.cancel_exec_button);
        btn.setOnClickListener(this);

        restoreInstanceState(view, savedInstanceState, CancelEditTextAndSwitchCombination.values());
    }

    @Override
    public void onSaveInstanceState(@NonNull Bundle outState) {
        super.onSaveInstanceState(outState);
        saveInstanceState(outState, CancelEditTextAndSwitchCombination.values());
    }

    @Override
    public void onClick(View v) {

        ComponentActivity activity = this.getActivity();
        if (activity == null) {
            return;
        }

        if (v.getId() == R.id.cancel_exec_button) {
            int callbackActivityFlag = Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_SINGLE_TOP;

            MposIntegrationService client = newMposIntegrationService(MposIntegrationAction.CANCEL);
            client.setCallbackPackageName(activity.getPackageName());
            client.setCallbackSuccessClassName(MainActivity.class);
            client.setCallbackErrorClassName(MainActivity.class);
            client.setCallbackCancelClassName(MainActivity.class);
            client.addCallbackSuccessActivityFlag(callbackActivityFlag);
            client.addCallbackErrorActivityFlag(callbackActivityFlag);
            client.addCallbackCancelActivityFlag(callbackActivityFlag);
            client.setOnNotStartActivityListener(this);

            // 連携を開始する。
            Intent intent = client.startMposApplication(activity, createMposIntegrationExtendedData());

            // 連携内容を保持する。
            IntegrationMessageContext.getInstance()
                    .addMessage(IntentUtil.extrasToTextValue(intent));
        }
    }

    @Override
    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {

        CancelEditTextAndSwitchCombination comb = CancelEditTextAndSwitchCombination.getBySwitchId(buttonView.getId());
        if (comb != null) {
            setTextViewEnabled(getView(), comb.getEditTextId(), isChecked);
        }
    }

    @Override
    Bundle createMposIntegrationExtendedData() {

        Bundle bundle = super.createMposIntegrationExtendedData();
        putBundleData(bundle, MposIntegrationKey.ORDER_ID, CancelEditTextAndSwitchCombination.ORDER_ID);
        putBundleData(bundle, MposIntegrationKey.ORIGINAL_ORDER_ID, CancelEditTextAndSwitchCombination.ORIGINAL_ORDER_ID);

        return bundle;
    }

    /**
     * {@link CancelFragment}におけるエディットテキストとスイッチの組み合わせを定義した列挙子。
     */
    private enum CancelEditTextAndSwitchCombination implements EditTextAndSwitchCombination {

        /** mPOS取引ID */
        ORDER_ID(R.id.cancel_order_id_text, R.id.cancel_order_id_switch),
        /** お客様アプリ取引ID */
        ORIGINAL_ORDER_ID(R.id.cancel_original_order_id_text, R.id.cancel_original_order_id_switch);

        /** 入力欄のID */
        private final int editTextId;
        /** スイッチのID */
        private final int switchId;

        /**
         * コンストラクタ。
         *
         * @param editTextId エディットテキストのID
         * @param switchId スイッチのID
         */
        CancelEditTextAndSwitchCombination(int editTextId, int switchId) {
            this.editTextId = editTextId;
            this.switchId = switchId;
        }

        @Override
        public int getEditTextId() {
            return editTextId;
        }

        @Override
        public int getSwitchId() {
            return switchId;
        }

        /**
         * 指定したスイッチのIDに対応する{@link CancelEditTextAndSwitchCombination}を取得する。<br>
         * 対応する{@link CancelEditTextAndSwitchCombination}が存在しない場合は{@code null}を取得する。
         *
         * @param switchId スイッチのID
         * @return {@link CancelEditTextAndSwitchCombination}
         */
        static CancelEditTextAndSwitchCombination getBySwitchId(int switchId) {

            for (CancelEditTextAndSwitchCombination e : CancelEditTextAndSwitchCombination.values()) {
                if (switchId == e.getSwitchId()) {
                    return e;
                }
            }
            return null;
        }
    }
}
