package jp.veritrans.mpos.sample.integration.ui;

import android.app.Activity;
import android.content.ClipData;
import android.content.ClipboardManager;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

import com.google.android.material.snackbar.Snackbar;

import jp.veritrans.mpos.sample.integration.R;
import jp.veritrans.mpos.sample.integration.ui.support.IntegrationMessageContext;

/**
 * mPOSアプリへのデータの送受信結果（メッセージ）を表示するフラグメント。
 */
@SuppressWarnings("WeakerAccess")
public class LogFragment extends Fragment implements AdapterView.OnItemLongClickListener {

    /** {@link ArrayAdapter} */
    private ArrayAdapter<String> adapter;

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        super.onCreateView(inflater, container, savedInstanceState);

        return inflater.inflate(R.layout.fragment_log, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        adapter = new ArrayAdapter<>(getActivity(), android.R.layout.simple_list_item_1, android.R.id.text1);
        ListView listView = view.findViewById(R.id.log_list_view);
        listView.setAdapter(adapter);
        listView.setOnItemLongClickListener(this);
    }

    @Override
    public void onResume() {
        super.onResume();

        adapter.clear();

        String[] messages = IntegrationMessageContext.getInstance().getMessages();
        for (int i = messages.length - 1; 0 <= i; i--) {
            adapter.add(messages[i]);
        }
    }

    @Override
    public boolean onItemLongClick(AdapterView<?> parent, View view, int position, long id) {

        Activity activity = getActivity();
        if (activity == null) {
            return false;
        }

        // クリップボードマネージャを取得する。
        ClipboardManager cm = (ClipboardManager) activity.getSystemService(Activity.CLIPBOARD_SERVICE);
        if (cm == null) {
            return false;
        }

        // クリップボードのコピーする。
        ListView listView = (ListView) parent;
        String msg = (String) listView.getItemAtPosition(position);
        ClipData clipData = ClipData.newPlainText("mpos_message", msg);
        cm.setPrimaryClip(clipData);

        Toast.makeText(activity, getString(R.string.msg_copied_to_clipboard), Toast.LENGTH_SHORT).show();
        return true;
    }

    /**
     * リストビューに表示したメッセージを全てクリアする。
     */
    public void clearMessageListView() {
        IntegrationMessageContext.getInstance().clearMessages();
        adapter.clear();

        View view = getView();
        if (view == null) {
            return;
        }
        Snackbar.make(view, getString(R.string.msg_clear_log_list), Snackbar.LENGTH_SHORT)
                .setAction("Action", null).show();
    }
}
