package jp.veritrans.mpos.sample.integration.ui;

import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.WindowManager;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;
import androidx.viewpager2.adapter.FragmentStateAdapter;
import androidx.viewpager2.widget.ViewPager2;

import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.google.android.material.tabs.TabLayout;
import com.google.android.material.tabs.TabLayoutMediator;

import java.util.HashMap;
import java.util.Map;

import jp.veritrans.mpos.sample.integration.R;
import jp.veritrans.mpos.sample.integration.ui.support.IntegrationMessageContext;
import jp.veritrans.mpos.sample.integration.ui.support.IntentDataDialogBuilder;
import jp.veritrans.mpos.sample.integration.util.IntentUtil;
import jp.veritrans.mpos.v2.integration.MposIntegrationAction;
import jp.veritrans.mpos.v2.integration.MposIntegrationKey;
import jp.veritrans.mpos.v2.integration.MposIntegrationService;

/**
 * メインとなるアクティビティ。
 */
public class MainActivity extends AppCompatActivity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setTitle(getString(R.string.app_name));
        setContentView(R.layout.activity_main);

        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        final SectionsPagerAdapter sectionsPagerAdapter = new SectionsPagerAdapter(this);
        final ViewPager2 viewPager = findViewById(R.id.container);
        viewPager.setAdapter(sectionsPagerAdapter);

        final TabLayout tabLayout = findViewById(R.id.tabs);
        new TabLayoutMediator(tabLayout, viewPager,
                (tab, position) -> {
                    CharSequence pageTitle;
                    if (position == SectionsPagePosition.PAYMENT.getPosition()) {
                        pageTitle = MainActivity.this.getString(R.string.tab_page_title_payment);

                    } else if (position == SectionsPagePosition.CANCEL.getPosition()) {
                        pageTitle = MainActivity.this.getString(R.string.tab_page_title_cancel);

                    } else if (position == SectionsPagePosition.REFUND.getPosition()) {
                        pageTitle = MainActivity.this.getString(R.string.tab_page_title_refund);

                    } else if (position == SectionsPagePosition.LOG.getPosition()) {
                        pageTitle = MainActivity.this.getString(R.string.tab_page_title_log);

                    } else {
                        pageTitle = MainActivity.this.getString(R.string.tab_page_title_payment);
                    }
                    tab.setText(pageTitle);
                }
        ).attach();

        final FloatingActionButton trashFab = findViewById(R.id.trash_fab);
        trashFab.hide();
        tabLayout.setEnabled(false);
        trashFab.setOnClickListener(view -> {
            Fragment fragment = sectionsPagerAdapter.getFragment(SectionsPagePosition.LOG.getPosition());
            if (fragment instanceof LogFragment) {
                ((LogFragment) fragment).clearMessageListView();
            }
        });

        viewPager.registerOnPageChangeCallback(new ViewPager2.OnPageChangeCallback() {
            @Override
            public void onPageSelected(int position) {
                if (position == SectionsPagePosition.LOG.getPosition()) {
                    trashFab.show();
                    trashFab.setEnabled(true);

                } else {
                    trashFab.hide();
                    trashFab.setEnabled(false);
                }
            }
        });

        getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);
    }

    @Override
    public void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        setIntent(intent);
    }

    @Override
    public void onResume() {
        super.onResume();

        if (MposIntegrationService.isCallingMposActivity(this)) {
            Intent intent = getIntent();
            int position;

            // 連携結果を保持する。
            MposIntegrationKey integrationKey = MposIntegrationKey.getInstance();
            String action = intent.getStringExtra(MposIntegrationKey.ACTION);
            if (MposIntegrationAction.PAYMENT.getType().equals(action)) {
                IntegrationMessageContext.getInstance().addMessage(//
                        IntentUtil.extrasToTextValue(intent, integrationKey.getPaymentResponseKeys()));
                position = SectionsPagePosition.PAYMENT.getPosition();

            } else if (MposIntegrationAction.CANCEL.getType().equals(action)) {
                IntegrationMessageContext.getInstance().addMessage(//
                        IntentUtil.extrasToTextValue(intent, integrationKey.getCancelResponseKeys()));
                position = SectionsPagePosition.CANCEL.getPosition();

            } else if (MposIntegrationAction.REFUND.getType().equals(action)) {
                IntegrationMessageContext.getInstance().addMessage(//
                        IntentUtil.extrasToTextValue(intent, integrationKey.getRefundResponseKeys()));
                position = SectionsPagePosition.REFUND.getPosition();

            } else {
                IntegrationMessageContext.getInstance().addMessage(//
                        IntentUtil.extrasToTextValue(intent));
                position = SectionsPagePosition.PAYMENT.getPosition();
            }

            ViewPager2 viewPager = findViewById(R.id.container);
            if (viewPager != null && viewPager.getCurrentItem() != position) {
                viewPager.setCurrentItem(position, true);
            }

            // 連携結果をダイアログ表示する。
            IntentDataDialogBuilder builder = new IntentDataDialogBuilder(this);
            this.setIntent(null);
            builder.create().show();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.option_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        boolean result = false;
        int itemId = item.getItemId();
        if (itemId == R.id.option_menu_version_item) {
            ApplicationVersionDialogFragment dialog = new ApplicationVersionDialogFragment();
            dialog.show(getSupportFragmentManager(), "version");
            result = true;
        }
        return result;
    }

    /**
     * ページ位置を定義した列挙子。
     */
    private enum SectionsPagePosition {

        /** 決済処理 */
        PAYMENT(0),
        /** 取消処理 */
        CANCEL(1),
        /** 返金処理 */
        REFUND(2),
        /** ログ表示 */
        LOG(3);

        /** ページ位置 */
        private final int position;

        /**
         * コンストラクタ。
         *
         * @param position ページ位置
         */
        SectionsPagePosition(int position) {
            this.position = position;
        }

        /**
         * ページ位置を取得する。
         *
         * @return ページ位置
         */
        int getPosition() {
            return position;
        }
    }

    /**
     * A {@link FragmentStateAdapter} that returns a fragment corresponding to one of the sections/tabs/pages.
     */
    private static class SectionsPagerAdapter extends FragmentStateAdapter {

        /** {@link Fragment}を格納した{@link Map} */
        private final Map<Integer, Fragment> fragments = new HashMap<>();

        /**
         * コンストラクタ。
         *
         * @param fa {@link FragmentActivity}
         */
        SectionsPagerAdapter(@NonNull FragmentActivity fa) {
            super(fa);
        }

        @NonNull
        @Override
        public Fragment createFragment(int position) {

            Fragment fragment;
            if (position == SectionsPagePosition.PAYMENT.getPosition()) {
                fragment = new PaymentFragment();

            } else if (position == SectionsPagePosition.CANCEL.getPosition()) {
                fragment = new CancelFragment();

            } else if (position == SectionsPagePosition.REFUND.getPosition()) {
                fragment = new RefundFragment();

            } else if (position == SectionsPagePosition.LOG.getPosition()) {
                fragment = new LogFragment();

            } else {
                fragment = new PaymentFragment();
            }
            fragments.put(position, fragment);
            return fragment;
        }

        @Override
        public int getItemCount() {
            return SectionsPagePosition.values().length;
        }

        /**
         * 指定した位置の{@link Fragment}を取得する。
         *
         * @param position 位置
         * @return {@link Fragment}
         */
        Fragment getFragment(int position) {
            return fragments.get(position);
        }
    }
}
