package jp.veritrans.mpos.sample.integration.ui;

import android.content.Context;
import android.util.AttributeSet;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ListAdapter;
import android.widget.ListView;

/**
 * 再計測を行うリストビュー。
 */
public class MeasuredListView extends ListView {

    /** 最大表示サイズ */
    private static final int MAX_SIZE = 100;

    /**
     * コンストラクタ。
     *
     * @param context {@link Context}
     * @param attrs {@link AttributeSet}
     */
    public MeasuredListView(Context context, AttributeSet attrs) {
        super(context, attrs);
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec);

        final int heightMode = MeasureSpec.getMode(heightMeasureSpec);
        if (heightMode == MeasureSpec.EXACTLY) {
            setMeasuredDimension(getMeasuredWidth(), getMeasuredHeight());
            return;
        }

        int newHeight = 0;
        int heightSize = MeasureSpec.getSize(heightMeasureSpec);

        ListAdapter listAdapter = getAdapter();
        if (listAdapter != null && !listAdapter.isEmpty()) {
            int listPosition = 0;
            for ( ; listPosition < listAdapter.getCount() && listPosition < MAX_SIZE; listPosition++) {
                View listItem = listAdapter.getView(listPosition, null, this);
                if (listItem instanceof ViewGroup) {
                    listItem.getLayoutParams().width = LayoutParams.WRAP_CONTENT;
                    listItem.getLayoutParams().height = LayoutParams.WRAP_CONTENT;
                }
                listItem.measure(widthMeasureSpec, heightMeasureSpec);
                newHeight += listItem.getMeasuredHeight();
            }
            newHeight += getDividerHeight() * listPosition;
        }

        if (heightMode == MeasureSpec.AT_MOST && heightSize < newHeight) {
            newHeight = heightSize;
        }
        setMeasuredDimension(getMeasuredWidth(), newHeight);
    }
}