package jp.veritrans.mpos.sample.integration.ui;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CompoundButton;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.ComponentActivity;

import jp.veritrans.mpos.sample.integration.R;
import jp.veritrans.mpos.sample.integration.ui.support.IntegrationMessageContext;
import jp.veritrans.mpos.sample.integration.util.IntentUtil;
import jp.veritrans.mpos.v2.integration.MposIntegrationAction;
import jp.veritrans.mpos.v2.integration.MposIntegrationKey;
import jp.veritrans.mpos.v2.integration.MposIntegrationService;

/**
 * mPOSアプリの決済処理と連携するフラグメント。
 */
public class PaymentFragment extends IntegrationBaseFragment
        implements View.OnClickListener, CompoundButton.OnCheckedChangeListener {

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        super.onCreateView(inflater, container, savedInstanceState);

        return inflater.inflate(R.layout.fragment_payment, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        Button btn = view.findViewById(R.id.payment_exec_button);
        btn.setOnClickListener(this);

        restoreInstanceState(view, savedInstanceState, PaymentEditTextAndSwitchCombination.values());
    }

    @Override
    public void onSaveInstanceState(@NonNull Bundle outState) {
        super.onSaveInstanceState(outState);
        saveInstanceState(outState, PaymentEditTextAndSwitchCombination.values());
    }

    @Override
    public void onClick(View v) {

        ComponentActivity activity = this.getActivity();
        if (activity == null) {
            return;
        }

        if (v.getId() == R.id.payment_exec_button) {
            int callbackActivityFlag = Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_SINGLE_TOP;

            MposIntegrationService client = newMposIntegrationService(MposIntegrationAction.PAYMENT);
            client.setCallbackPackageName(activity.getPackageName());
            client.setCallbackSuccessClassName(MainActivity.class);
            client.setCallbackErrorClassName(MainActivity.class);
            client.setCallbackCancelClassName(MainActivity.class);
            client.addCallbackSuccessActivityFlag(callbackActivityFlag);
            client.addCallbackErrorActivityFlag(callbackActivityFlag);
            client.addCallbackCancelActivityFlag(callbackActivityFlag);
            client.setOnNotStartActivityListener(this);

            // 連携を開始する。
            Intent intent = client.startMposApplication(activity, createMposIntegrationExtendedData());

            // 連携内容を保持する。
            IntegrationMessageContext.getInstance()
                    .addMessage(IntentUtil.extrasToTextValue(intent));
        }
    }

    @Override
    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {

        PaymentEditTextAndSwitchCombination comb = PaymentEditTextAndSwitchCombination.getBySwitchId(buttonView.getId());
        if (comb != null) {
            setTextViewEnabled(requireView(), comb.getEditTextId(), isChecked);
        }
    }

    @Override
    Bundle createMposIntegrationExtendedData() {

        Bundle bundle = super.createMposIntegrationExtendedData();
        putBundleData(bundle, MposIntegrationKey.AMOUNT, PaymentEditTextAndSwitchCombination.AMOUNT);
        putBundleData(bundle, MposIntegrationKey.DESCRIPTION, PaymentEditTextAndSwitchCombination.DESCRIPTION);
        putBundleData(bundle, MposIntegrationKey.MILES, PaymentEditTextAndSwitchCombination.MILES);
        putBundleData(bundle, MposIntegrationKey.TIMES_MILES, PaymentEditTextAndSwitchCombination.TIMES_MILES);
        putBundleData(bundle, MposIntegrationKey.ORIGINAL_ORDER_ID, PaymentEditTextAndSwitchCombination.ORIGINAL_ORDER_ID);
        putBundleData(bundle, MposIntegrationKey.INPUT_AMOUNT_VIEW, PaymentEditTextAndSwitchCombination.INPUT_AMOUNT_VIEW);
        putBundleData(bundle, MposIntegrationKey.COMPLETE_VIEW, PaymentEditTextAndSwitchCombination.COMPLETE_VIEW);

        return bundle;
    }

    /**
     * {@link PaymentFragment}におけるエディットテキストとスイッチの組み合わせを定義した列挙子。
     */
    private enum PaymentEditTextAndSwitchCombination implements EditTextAndSwitchCombination {

        /** 金額 */
        AMOUNT(R.id.payment_amount_text, R.id.payment_amount_switch),
        /** 商品説明 */
        DESCRIPTION(R.id.payment_description_text, R.id.payment_description_switch),
        /** マイル数 */
        MILES(R.id.payment_miles_text, R.id.payment_miles_switch),
        /** マイル回数 */
        TIMES_MILES(R.id.payment_times_miles_text, R.id.payment_times_miles_switch),
        /** お客様アプリ取引ID */
        ORIGINAL_ORDER_ID(R.id.payment_original_order_id_text, R.id.payment_original_order_id_switch),
        /** 金額入力画面表示フラグ */
        INPUT_AMOUNT_VIEW(R.id.payment_input_amount_view_text, R.id.payment_input_amount_view_switch),
        /** 完了画面表示フラグ */
        COMPLETE_VIEW(R.id.payment_complete_view_text, R.id.payment_complete_view_switch);

        /** エディットテキストのID */
        private final int editTextId;
        /** スイッチのID */
        private final int switchId;

        /**
         * コンストラクタ。
         *
         * @param editTextId エディットテキストのID
         * @param switchId スイッチのID
         */
        PaymentEditTextAndSwitchCombination(int editTextId, int switchId) {
            this.editTextId = editTextId;
            this.switchId = switchId;
        }

        @Override
        public int getEditTextId() {
            return editTextId;
        }

        @Override
        public int getSwitchId() {
            return switchId;
        }

        /**
         * 指定したスイッチのIDに対応する{@link PaymentEditTextAndSwitchCombination}を取得する。<br>
         * 対応する{@link PaymentEditTextAndSwitchCombination}が存在しない場合は{@code null}を取得する。
         *
         * @param switchId スイッチのID
         * @return {@link PaymentEditTextAndSwitchCombination}
         */
        static PaymentEditTextAndSwitchCombination getBySwitchId(int switchId) {

            for (PaymentEditTextAndSwitchCombination e : PaymentEditTextAndSwitchCombination.values()) {
                if (switchId == e.getSwitchId()) {
                    return e;
                }
            }
            return null;
        }
    }
}
