package jp.veritrans.mpos.sample.integration.ui;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CompoundButton;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.ComponentActivity;

import jp.veritrans.mpos.sample.integration.R;
import jp.veritrans.mpos.sample.integration.ui.support.IntegrationMessageContext;
import jp.veritrans.mpos.sample.integration.util.IntentUtil;
import jp.veritrans.mpos.v2.integration.MposIntegrationAction;
import jp.veritrans.mpos.v2.integration.MposIntegrationKey;
import jp.veritrans.mpos.v2.integration.MposIntegrationService;

/**
 * mPOSアプリの返金処理と連携するフラグメント。
 */
public class RefundFragment extends IntegrationBaseFragment
        implements View.OnClickListener, CompoundButton.OnCheckedChangeListener {

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        super.onCreateView(inflater, container, savedInstanceState);

        return inflater.inflate(R.layout.fragment_refund, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        Button btn = view.findViewById(R.id.refund_exec_button);
        btn.setOnClickListener(this);

        restoreInstanceState(view, savedInstanceState, RefundEditTextAndSwitchCombination.values());
    }

    @Override
    public void onSaveInstanceState(@NonNull Bundle outState) {
        super.onSaveInstanceState(outState);
        saveInstanceState(outState, RefundEditTextAndSwitchCombination.values());
    }

    @Override
    public void onClick(View v) {

        ComponentActivity activity = this.getActivity();
        if (activity == null) {
            return;
        }

        if (v.getId() == R.id.refund_exec_button) {
            int callbackActivityFlag = Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_SINGLE_TOP;

            MposIntegrationService client = newMposIntegrationService(MposIntegrationAction.REFUND);
            client.setCallbackPackageName(activity.getPackageName());
            client.setCallbackSuccessClassName(MainActivity.class);
            client.setCallbackErrorClassName(MainActivity.class);
            client.setCallbackCancelClassName(MainActivity.class);
            client.addCallbackSuccessActivityFlag(callbackActivityFlag);
            client.addCallbackErrorActivityFlag(callbackActivityFlag);
            client.addCallbackCancelActivityFlag(callbackActivityFlag);
            client.setOnNotStartActivityListener(this);

            // 連携を開始する。
            Intent intent = client.startMposApplication(activity, createMposIntegrationExtendedData());

            // 連携内容を保持する。
            IntegrationMessageContext.getInstance()
                    .addMessage(IntentUtil.extrasToTextValue(intent));
        }
    }

    @Override
    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {

        RefundEditTextAndSwitchCombination comb = RefundEditTextAndSwitchCombination.getBySwitchId(buttonView.getId());
        if (comb != null) {
            setTextViewEnabled(getView(), comb.getEditTextId(), isChecked);
        }
    }

    @Override
    Bundle createMposIntegrationExtendedData() {

        Bundle bundle = super.createMposIntegrationExtendedData();
        putBundleData(bundle, MposIntegrationKey.ORDER_ID, RefundEditTextAndSwitchCombination.ORDER_ID);
        putBundleData(bundle, MposIntegrationKey.ORIGINAL_ORDER_ID, RefundEditTextAndSwitchCombination.ORIGINAL_ORDER_ID);

        return bundle;
    }

    /**
     * {@link RefundFragment}におけるエディットテキストとスイッチの組み合わせを定義した列挙子。
     */
    private enum RefundEditTextAndSwitchCombination implements EditTextAndSwitchCombination {

        /** mPOS取引ID */
        ORDER_ID(R.id.refund_order_id_text, R.id.refund_order_id_switch),
        /** お客様アプリ取引ID */
        ORIGINAL_ORDER_ID(R.id.refund_original_order_id_text, R.id.refund_original_order_id_switch);

        /** 入力欄のID */
        private final int editTextId;
        /** スイッチのID */
        private final int switchId;

        /**
         * コンストラクタ。
         *
         * @param editTextId エディットテキストのID
         * @param switchId スイッチのID
         */
        RefundEditTextAndSwitchCombination(int editTextId, int switchId) {
            this.editTextId = editTextId;
            this.switchId = switchId;
        }

        @Override
        public int getEditTextId() {
            return editTextId;
        }

        @Override
        public int getSwitchId() {
            return switchId;
        }

        /**
         * 指定したスイッチのIDに対応する{@link RefundEditTextAndSwitchCombination}を取得する。<br>
         * 対応する{@link RefundEditTextAndSwitchCombination}が存在しない場合は{@code null}を取得する。
         *
         * @param switchId スイッチのID
         * @return {@link RefundEditTextAndSwitchCombination}
         */
        static RefundEditTextAndSwitchCombination getBySwitchId(int switchId) {

            for (RefundEditTextAndSwitchCombination e : RefundEditTextAndSwitchCombination.values()) {
                if (switchId == e.getSwitchId()) {
                    return e;
                }
            }
            return null;
        }
    }
}
