package jp.veritrans.mpos.sample.integration.ui.support;

import java.util.ArrayList;
import java.util.List;

/**
 * mPOSアプリへのデータの送受信結果（メッセージ）を格納するコンテキスト。
 */
public class IntegrationMessageContext {

    /** コンテキスト */
    private static final IntegrationMessageContext CONTEXT = new IntegrationMessageContext();

    /** メッセージリスト */
    private final List<String> messageList = new ArrayList<>();

    /**
     * コンストラクタ。
     */
    private IntegrationMessageContext() {
    }

    /**
     * インスタンスを取得する。
     *
     * @return {@link IntegrationMessageContext}
     */
    public static IntegrationMessageContext getInstance() {
        return CONTEXT;
    }

    /**
     * メッセージを追加する。
     *
     * @param message メッセージ
     */
    public void addMessage(String message) {

        if (message == null || message.isEmpty()) {
            return;
        }
        messageList.add(message);
    }

    /**
     * メッセージをクリアする。
     */
    public void clearMessages() {
        messageList.clear();
    }

    /**
     * コンテキストが保持するすべてのメッセージを取得する。
     *
     * @return メッセージの配列
     */
    public String[] getMessages() {
        return messageList.toArray(new String[0]);
    }
}
