package jp.veritrans.mpos.sample.integration.ui.support;

import android.content.Intent;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;

import jp.veritrans.mpos.sample.integration.util.IntentUtil;
import jp.veritrans.mpos.v2.integration.MposIntegrationAction;
import jp.veritrans.mpos.v2.integration.MposIntegrationKey;

/**
 * mPOSアプリとの連携に使用した{@link Intent}の内容をダイアログ表示するためのビルダークラス。
 */
public class IntentDataDialogBuilder {

    /** {@link AppCompatActivity} */
    private final AppCompatActivity activity;
    /** mPOSアプリ連携データが格納された{@link Intent} */
    private final Intent intent;

    /**
     * コンストラクタ。
     *
     * @param activity {@link AppCompatActivity}
     */
    public IntentDataDialogBuilder(AppCompatActivity activity) {
        this.activity = activity;
        this.intent = activity.getIntent();
    }

    /**
     * {@link AlertDialog}を生成する。
     *
     * @return {@link AlertDialog}
     */
    public AlertDialog create() {

        AlertDialog.Builder alertDialog = new AlertDialog.Builder(activity);
        alertDialog.setTitle("結果");
        alertDialog.setMessage(createMessage());
        alertDialog.setPositiveButton("OK", (dialog, which) -> {
        });
        return alertDialog.create();
    }

    /**
     * メッセージを生成する。
     *
     * @return メッセージ
     */
    private String createMessage() {

        if (intent == null) {
            return "";
        }

        String result;

        MposIntegrationKey integrationKey = MposIntegrationKey.getInstance();
        String action = intent.getStringExtra(MposIntegrationKey.ACTION);
        if (MposIntegrationAction.PAYMENT.getType().equals(action)) {
            result = IntentUtil.extrasToTextValue(intent, integrationKey.getPaymentResponseKeys());

        } else if (MposIntegrationAction.REFUND.getType().equals(action)) {
            result = IntentUtil.extrasToTextValue(intent, integrationKey.getRefundResponseKeys());

        } else {
            result = IntentUtil.extrasToTextValue(intent);
        }
        return result;
    }
}
