package jp.veritrans.mpos.sample.integration.util;

import android.content.Intent;
import android.os.Bundle;

/**
 * {@link Intent}に関するユーティリティクラス。
 */
public final class IntentUtil {

    /**
     * コンストラクタ。
     */
    private IntentUtil() {
    }

    /**
     * 指定した{@link Intent}に格納された拡張データを表示用の文字列に変換する。
     *
     * @param intent {@link Intent}
     * @return 表示用の文字列
     */
    public static String extrasToTextValue(Intent intent) {

        if (intent == null) {
            return "";
        }

        Bundle bundle = intent.getExtras();
        if (bundle == null) {
            return "";
        }

        StringBuilder sb = new StringBuilder();
        for (String key : bundle.keySet()) {
            sb.append(key).append("=").append(intent.getSerializableExtra(key)).append("\n");
        }
        return sb.toString();
    }

    /**
     * 指定した{@link Intent}に格納された拡張データを表示用の文字列に変換する。<br>
     * 文字列に変換する対象となる拡張データは{@code extraKeys}で指定した項目となる。
     *
     * @param intent {@link Intent}
     * @param extraKeys 対象とする拡張データのキーを指定した配列
     * @return 表示用の文字列
     */
    public static String extrasToTextValue(Intent intent, String[] extraKeys) {

        if (intent == null || extraKeys == null) {
            return "";
        }

        StringBuilder sb = new StringBuilder();
        for (String key : extraKeys) {
            if (!intent.hasExtra(key)) {
                continue;
            }
            sb.append(key).append("=").append(intent.getSerializableExtra(key)).append("\n");
        }
        return sb.toString();
    }
}
