package jp.veritrans.mpos.v2.integration;

import java.util.Arrays;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

/**
 * mPOSアプリとの連携で利用するパラメータキーを定義したクラス。
 */
@SuppressWarnings({"WeakerAccess", "unused"})
public final class MposIntegrationKey implements MposIntegrationKeyProvider {

    /** アクション */
    public static final String ACTION = "action";
    /** コールバックパッケージ名 */
    public static final String CALLBACK_PACKAGE_NAME = "callbackPackageName";
    /** コールバック成功クラス名 */
    public static final String CALLBACK_SUCCESS_CLASS_NAME = "callbackSuccessClassName";
    /** コールバック失敗クラス名 */
    public static final String CALLBACK_ERROR_CLASS_NAME = "callbackErrorClassName";
    /** コールバック中止クラス名 */
    public static final String CALLBACK_CANCEL_CLASS_NAME = "callbackCancelClassName";
    /** コールバック成功起動フラグ */
    public static final String CALLBACK_SUCCESS_ACTIVITY_FLAG = "callbackSuccessActivityFlag";
    /** コールバック失敗起動フラグ */
    public static final String CALLBACK_ERROR_ACTIVITY_FLAG = "callbackErrorActivityFlag";
    /** コールバック中止起動フラグ */
    public static final String CALLBACK_CANCEL_ACTIVITY_FLAG = "callbackCancelActivityFlag";
    /** 遷移先のアクティビティを呼び出した側のアプリパッケージ名（mPOSアプリのパッケージ名） */
    public static final String CALLING_PACKAGE_NAME = "callingPackageName";
    /** マイル数 */
    public static final String MILES = "miles";
    /** マイル回数 */
    public static final String TIMES_MILES = "timesMiles";
    /** 金額入力画面表示フラグ */
    public static final String INPUT_AMOUNT_VIEW = "inputAmountView";
    /** 完了画面表示フラグ */
    public static final String COMPLETE_VIEW = "completeView";
    /** 処理ステータス */
    public static final String STATUS = "status";
    /** エラーコード */
    public static final String ERROR_CODE = "errorCode";
    /** エラーメッセージ */
    public static final String ERROR_MESSAGE = "errorMessage";
    /** 結果コード */
    public static final String RESULT_CODE = "resultCode";
    /** mPOS取引ID */
    public static final String ORDER_ID = "orderId";
    /** mPOS取引照会ID */
    public static final String ORDER_REFERENCE_ID = "orderReferenceId";
    /** お客様アプリ取引ID */
    public static final String ORIGINAL_ORDER_ID = "originalOrderId";
    /** 決済方法コード */
    public static final String PAYMENT_METHOD_CODE = "paymentMethodCode";
    /** 支払手段 */
    public static final String PAYMENT_SOURCE = "paymentSource";
    /** 決済金額 */
    public static final String AMOUNT = "amount";
    /** 割引金額 */
    public static final String DISCOUNT_AMOUNT = "discountAmount";
    /** 合計金額 */
    public static final String TOTAL_AMOUNT = "totalAmount";
    /** 取引日時 */
    public static final String PAYMENT_TIME = "paymentTime";
    /** 説明 */
    public static final String DESCRIPTION = "description";
    /** カード番号 */
    public static final String CARD_NUMBER = "cardNumber";
    /** カード番号トークン */
    public static final String CARD_NUMBER_TOKEN = "cardNumberToken";
    /** カード有効期限 */
    public static final String CARD_EXPIRY = "cardExpiry";
    /** カードタイプ */
    public static final String CARD_TYPE = "cardType";
    /** 支払種別 */
    public static final String JPO = "jpo";
    /** 承認番号 */
    public static final String AUTH_CODE = "authCode";
    /** 仕向け先コード	 */
    public static final String ACQUIRER_CODE = "acquirerCode";
    /** 決済センターアクワイアラID */
    public static final String CENTER_ACQUIRER_ID = "centerAcquirerId";
    /** 決済センター取引ID */
    public static final String CENTER_TRADE_ID = "centerTradeId";
    /** 決済センター処理ID */
    public static final String CENTER_PROCESS_ID = "centerProcessId";
    /** 決済センター照会コード */
    public static final String CENTER_REFERENCE_CODE = "centerReferenceCode";
    /** 決済センター取引タイプ */
    public static final String CENTER_TRANSACTION_TYPE = "centerTransactionType";
    /** 決済センター取引日時 */
    public static final String CENTER_TRANSACTION_DATETIME = "centerTransactionDatetime";
    /** 決済センター返金日時 */
    public static final String CENTER_REFUND_DATETIME = "centerRefundDatetime";
    /** 決済センター端末ID */
    public static final String CENTER_TERMINAL_ID = "centerTerminalId";
    /** 決済センターマーチャントID */
    public static final String CENTER_MERCHANT_ID = "centerMerchantId";
    /** 決済センターマーチャント名 */
    public static final String CENTER_MERCHANT_NAME = "centerMerchantName";
    /** 銀聯要求送信日時 */
    public static final String CUP_REQUEST_TIME = "cupRequestTime";
    /** EMVアプリケーションID */
    public static final String EMV_APP_ID = "emvAppId";
    /** EMVアプリケーショントランザクションカウンター */
    public static final String EMV_ATC = "emvAtc";
    /** EMアプリケーションラベル */
    public static final String EMV_APP_LABEL = "emvAppLabel";
    /** カードシーケンス番号 */
    public static final String PAN_SEQ_NUMBER = "panSeqNumber";
    /** カード所有者検証方法タイプ */
    public static final String CVM_TYPE = "cvmType";
    /** カード所有者検証結果 */
    public static final String CVMR = "cvmr";
    /** 端末検証結果 */
    public static final String TVR = "tvr";
    /** トランザクションステータス情報 */
    public static final String TSI = "tsi";
    /** トランザクション暗号文 */
    public static final String TXN_CRYPTOGRAM = "txnCryptogram";
    /** マイレージ会員番号用 */
    public static final String MILEAGE_MEMBERSHIP_NO = "mileageMembershipNo";
    /** 利用タイプ */
    public static final String USAGE_TYPE = "usageType";
    /** 前回利用日 */
    public static final String LAST_USE_DATE = "lastUseDate";

    /** インスタンス */
    private static final MposIntegrationKey INSTANCE = new MposIntegrationKey();
    /** 追加パラメーターキープロバイダー */
    private static final MposIntegrationKeyProvider DEFAULT_ADDITIONAL_KEY_PROVIDER = new DefaultAdditionalIntegrationKeyProvider();

    /** {@link MposIntegrationKeyProvider} */
    private static MposIntegrationKeyProvider additionalKeyProvider = DEFAULT_ADDITIONAL_KEY_PROVIDER;

    /**
     * コンストラクタ。
     */
    private MposIntegrationKey() {
    }

    /**
     * {@link MposIntegrationKey}のインスタンスを取得する。
     *
     * @return インスタンス
     */
    public static MposIntegrationKey getInstance() {
        return INSTANCE;
    }

    /**
     * 初期化を行う。
     *
     * @param keyProvider 追加パラメーターを提供する{@link MposIntegrationKeyProvider}
     */
    public static void initialize(MposIntegrationKeyProvider keyProvider) {

        if (keyProvider == null) {
            MposIntegrationKey.additionalKeyProvider = DEFAULT_ADDITIONAL_KEY_PROVIDER;

        } else {
            MposIntegrationKey.additionalKeyProvider = keyProvider;
        }
    }

    /**
     * 決済処理要求のパラメーターキーを取得する。
     *
     * @return パラメーターキー
     */
    public String[] getPaymentRequestKeys() {

        Set<String> keys = new LinkedHashSet<>();
        keys.add(ACTION);
        keys.add(AMOUNT);
        keys.add(DESCRIPTION);
        keys.add(INPUT_AMOUNT_VIEW);
        keys.add(COMPLETE_VIEW);
        keys.add(ORIGINAL_ORDER_ID);
        keys.add(CALLBACK_PACKAGE_NAME);
        keys.add(CALLBACK_SUCCESS_CLASS_NAME);
        keys.add(CALLBACK_ERROR_CLASS_NAME);
        keys.add(CALLBACK_CANCEL_CLASS_NAME);
        keys.add(CALLBACK_SUCCESS_ACTIVITY_FLAG);
        keys.add(CALLBACK_ERROR_ACTIVITY_FLAG);
        keys.add(CALLBACK_CANCEL_ACTIVITY_FLAG);

        keys.addAll(asList(additionalKeyProvider.getPaymentRequestKeys()));

        return keys.toArray(new String[0]);
    }

    /**
     * 決済処理応答のパラメーターキーを取得する。
     *
     * @return パラメーターキー
     */
    public String[] getPaymentResponseKeys() {

        Set<String> keys = new LinkedHashSet<>();
        keys.add(ACTION);
        keys.add(STATUS);
        keys.add(CALLING_PACKAGE_NAME);
        keys.add(ERROR_CODE);
        keys.add(ERROR_MESSAGE);
        keys.add(RESULT_CODE);
        keys.add(ORDER_ID);
        keys.add(ORDER_REFERENCE_ID);
        keys.add(ORIGINAL_ORDER_ID);
        keys.add(PAYMENT_METHOD_CODE);
        keys.add(PAYMENT_SOURCE);
        keys.add(AMOUNT);
        keys.add(DISCOUNT_AMOUNT);
        keys.add(TOTAL_AMOUNT);
        keys.add(PAYMENT_TIME);
        keys.add(DESCRIPTION);
        keys.add(CARD_NUMBER);
        keys.add(CARD_NUMBER_TOKEN);
        keys.add(CARD_EXPIRY);
        keys.add(CARD_TYPE);
        keys.add(JPO);
        keys.add(AUTH_CODE);
        keys.add(ACQUIRER_CODE);
        keys.add(CENTER_ACQUIRER_ID);
        keys.add(CENTER_TRADE_ID);
        keys.add(CENTER_PROCESS_ID);
        keys.add(CENTER_REFERENCE_CODE);
        keys.add(CENTER_TRANSACTION_TYPE);
        keys.add(CENTER_TRANSACTION_DATETIME);
        keys.add(CENTER_TERMINAL_ID);
        keys.add(CENTER_MERCHANT_ID);
        keys.add(CENTER_MERCHANT_NAME);
        keys.add(CUP_REQUEST_TIME);
        keys.add(EMV_APP_ID);
        keys.add(EMV_ATC);
        keys.add(EMV_APP_LABEL);
        keys.add(PAN_SEQ_NUMBER);
        keys.add(CVM_TYPE);
        keys.add(CVMR);
        keys.add(TVR);
        keys.add(TSI);
        keys.add(TXN_CRYPTOGRAM);
        keys.add(MILEAGE_MEMBERSHIP_NO);
        keys.add(USAGE_TYPE);
        keys.add(LAST_USE_DATE);

        keys.addAll(asList(additionalKeyProvider.getPaymentResponseKeys()));

        return keys.toArray(new String[0]);
    }

    /**
     * 取消処理要求のパラメーターキーを取得する。
     *
     * @return パラメーターキー
     */
    public String[] getCancelRequestKeys() {

        Set<String> keys = new LinkedHashSet<>();
        keys.add(ACTION);
        keys.add(ORDER_ID);
        keys.add(ORIGINAL_ORDER_ID);
        keys.add(CALLBACK_PACKAGE_NAME);
        keys.add(CALLBACK_SUCCESS_CLASS_NAME);
        keys.add(CALLBACK_ERROR_CLASS_NAME);
        keys.add(CALLBACK_CANCEL_CLASS_NAME);
        keys.add(CALLBACK_SUCCESS_ACTIVITY_FLAG);
        keys.add(CALLBACK_ERROR_ACTIVITY_FLAG);
        keys.add(CALLBACK_CANCEL_ACTIVITY_FLAG);

        keys.addAll(asList(additionalKeyProvider.getCancelRequestKeys()));

        return keys.toArray(new String[0]);
    }

    /**
     * 取消処理応答のパラメーターキーを取得する。
     *
     * @return パラメーターキー
     */
    public String[] getCancelResponseKeys() {

        Set<String> keys = new LinkedHashSet<>();
        keys.add(ACTION);
        keys.add(STATUS);
        keys.add(CALLING_PACKAGE_NAME);
        keys.add(ERROR_CODE);
        keys.add(ERROR_MESSAGE);
        keys.add(RESULT_CODE);
        keys.add(ORDER_ID);
        keys.add(ORIGINAL_ORDER_ID);
        keys.add(CENTER_PROCESS_ID);
        keys.add(CUP_REQUEST_TIME);

        keys.addAll(asList(additionalKeyProvider.getCancelResponseKeys()));

        return keys.toArray(new String[0]);
    }

    /**
     * 返金処理要求のパラメーターキーを取得する。
     *
     * @return パラメーターキー
     */
    public String[] getRefundRequestKeys() {

        Set<String> keys = new LinkedHashSet<>();
        keys.add(ACTION);
        keys.add(ORDER_ID);
        keys.add(ORIGINAL_ORDER_ID);
        keys.add(CALLBACK_PACKAGE_NAME);
        keys.add(CALLBACK_SUCCESS_CLASS_NAME);
        keys.add(CALLBACK_ERROR_CLASS_NAME);
        keys.add(CALLBACK_CANCEL_CLASS_NAME);
        keys.add(CALLBACK_SUCCESS_ACTIVITY_FLAG);
        keys.add(CALLBACK_ERROR_ACTIVITY_FLAG);
        keys.add(CALLBACK_CANCEL_ACTIVITY_FLAG);

        keys.addAll(asList(additionalKeyProvider.getRefundRequestKeys()));

        return keys.toArray(new String[0]);
    }

    /**
     * 返金処理応答のパラメーターキーを取得する。
     *
     * @return パラメーターキー
     */
    public String[] getRefundResponseKeys() {

        Set<String> keys = new LinkedHashSet<>();
        keys.add(ACTION);
        keys.add(STATUS);
        keys.add(CALLING_PACKAGE_NAME);
        keys.add(ERROR_CODE);
        keys.add(ERROR_MESSAGE);
        keys.add(RESULT_CODE);
        keys.add(ORDER_ID);
        keys.add(ORIGINAL_ORDER_ID);
        keys.add(CENTER_ACQUIRER_ID);
        keys.add(CENTER_TRADE_ID);
        keys.add(CENTER_PROCESS_ID);
        keys.add(CENTER_REFERENCE_CODE);
        keys.add(CENTER_TRANSACTION_TYPE);
        keys.add(CENTER_TRANSACTION_DATETIME);
        keys.add(CENTER_REFUND_DATETIME);
        keys.add(CENTER_TERMINAL_ID);
        keys.add(CENTER_MERCHANT_ID);
        keys.add(CENTER_MERCHANT_NAME);
        keys.add(CUP_REQUEST_TIME);

        keys.addAll(asList(additionalKeyProvider.getRefundResponseKeys()));

        return keys.toArray(new String[0]);
    }

    /**
     * 配列をリストに変換する。
     *
     * @param array 配列
     * @return リスト
     */
    private List<String> asList(String[] array) {

        if (array == null || array.length == 0) {
            return Collections.emptyList();
        }
        return Arrays.asList(array);
    }

    /**
     * {@link MposIntegrationKeyProvider}を実装したデフォルトクラス。
     */
    private static class DefaultAdditionalIntegrationKeyProvider implements MposIntegrationKeyProvider {
    }
}
