package jp.veritrans.mpos.v2.integration;

/**
 * mPOSアプリとの連携で利用するパラメータキーを提供するインターフェース。
 */
interface MposIntegrationKeyProvider {

    /**
     * 決済処理要求のパラメーターキーを取得する。
     *
     * @return パラメーターキー
     */
    default String[] getPaymentRequestKeys() {
        return new String[0];
    }

    /**
     * 決済処理応答のパラメーターキーを取得する。
     *
     * @return パラメーターキー
     */
    default String[] getPaymentResponseKeys() {
        return new String[0];
    }

    /**
     * 取消処理要求のパラメーターキーを取得する。
     *
     * @return パラメーターキー
     */
    default String[] getCancelRequestKeys() {
        return new String[0];
    }

    /**
     * 取消処理応答のパラメーターキーを取得する。
     *
     * @return パラメーターキー
     */
    default String[] getCancelResponseKeys() {
        return new String[0];
    }

    /**
     * 返金処理要求のパラメーターキーを取得する。
     *
     * @return パラメーターキー
     */
    default String[] getRefundRequestKeys() {
        return new String[0];
    }

    /**
     * 返金処理応答のパラメーターキーを取得する。
     *
     * @return パラメーターキー
     */
    default String[] getRefundResponseKeys() {
        return new String[0];
    }
}
